import { __extends, __values, __decorate } from 'tslib';
import { EventEmitter, ElementRef, Input, Output, Component, NgModule } from '@angular/core';
import { _, findPlugin, assignGridOptions, assignDeep, setInstance, deleteInstance } from '@grapecity/dataviews.common';
import { DataView } from '@grapecity/dataviews.core';

/**
 * @module Angular
 *
 * This module provides NgModule and component for Angular integration of {@link DataView} control.
 */
function isTrue(val) {
    if (_.isNil(val)) {
        return false;
    }
    if (_.isBoolean(val)) {
        return val !== false;
    }
    if (_.isNumber(val)) {
        return val !== 0;
    }
    return true;
}
function makeGridLayout(options) {
    var Plugin = findPlugin('GridLayout');
    if (!Plugin) {
        throw new Error('@grapecity/dataviews.grid is not loaded');
    }
    return new Plugin(options);
}
var ɵ0 = function (v) { return v.selectionChanged; }, ɵ1 = function (v) { return v.focusChanged; }, ɵ2 = function (v) { return v.grouping; }, ɵ3 = function (v) { return v.rowClick; }, ɵ4 = function (v) { return v.rowDbClick; }, ɵ5 = function (v) { return v.columnReorder; }, ɵ6 = function (v) { return v.rowReorder; }, ɵ7 = function (v) { return v.columnChanged; }, ɵ8 = function (v) { return v.editing; }, ɵ9 = function (v) { return v.beforeLoadRange; };
var eventMap = {
    selectionChanged: ɵ0,
    focusChanged: ɵ1,
    grouping: ɵ2,
    rowClick: ɵ3,
    rowDbClick: ɵ4,
    columnReorder: ɵ5,
    rowReorder: ɵ6,
    columnChanged: ɵ7,
    editing: ɵ8,
    beforeLoadRange: ɵ9,
};
function isEmpty(a) {
    return !a || a.length === 0;
}
var GridEvent = /** @class */ (function (_super) {
    __extends(GridEvent, _super);
    function GridEvent(name, component) {
        var _this = _super.call(this) || this;
        _this.emitter = function (event) {
            _this.emit(event);
        };
        _this.name = name;
        _this.component = component;
        return _this;
    }
    GridEvent.prototype.subscribe = function (generatorOrNext, error, complete) {
        var _this = this;
        var onObservers = isEmpty(this.observers);
        var sub = _super.prototype.subscribe.call(this, generatorOrNext, error, complete);
        if (onObservers) {
            this.register();
        }
        sub.add(function () {
            if (isEmpty(_this.observers)) {
                _this.unregister();
            }
        });
        return sub;
    };
    Object.defineProperty(GridEvent.prototype, "gridEvent", {
        get: function () {
            return eventMap[this.name](this.component.dataView);
        },
        enumerable: true,
        configurable: true
    });
    GridEvent.prototype.register = function () {
        if (this.component.dataView && !this.gridEvent.contains(this.emitter)) {
            this.gridEvent.addHandler(this.emitter);
        }
    };
    GridEvent.prototype.unregister = function () {
        if (this.component.dataView) {
            this.gridEvent.removeHandler(this.emitter);
        }
    };
    return GridEvent;
}(EventEmitter));
/**
 * Angular component for {@link DataView} control
 *
 * See examples:
 * - {@sample Features/Basics/Introduction/angular Basic Data Grid}
 * - {@sample DataViews/Calendar/Calendar/angular Calendar Layout}
 * - {@sample DataViews/Card/CardLayoutEngine/angular Card Layout}
 * - {@sample DataViews/Trellis/TrellisWithGridLayout/angular Trellis Layout}
 * - {@sample DataViews/Masonry/MasonryLayout/angular Masonry Layout}
 * - {@sample DataViews/Timeline/TimelineGrouping/angular Timeline Grouping}
 */
var DataViewComponent = /** @class */ (function () {
    /**
     * Initializes new instance of DataViewComponent
     * @param container a reference to host container
     */
    function DataViewComponent(container) {
        this._initialized = false;
        /**
         * This event occurs on each creation of underlying DataView component.
         */
        this.created = new EventEmitter();
        /**
         * This event occurs when the user changes the selection using UI actions.
         */
        this.selectionChanged = new GridEvent('selectionChanged', this);
        /**
         * This event occurs when the focused row is changed. The event fires on [[GridLayout]] or [[HorizontalLayout]]
         */
        this.focusChanged = new GridEvent('focusChanged', this);
        /**
         * This event occurs when the user updates groups with UI actions. The event is fired before the actual update happens
         */
        this.grouping = new GridEvent('grouping', this);
        /**
         * This event occurs when clicking the row
         */
        this.rowClick = new GridEvent('rowClick', this);
        /**
         * This event occurs when double-clicking the row
         */
        this.rowDbClick = new GridEvent('rowDbClick', this);
        /**
         * This event occurs when the user starts dragging the column and before dropping the column in the column header
         */
        this.columnReorder = new GridEvent('columnReorder', this);
        /**
         * This event occurs when the user starts dragging the row and before inserting the row at new position
         */
        this.rowReorder = new GridEvent('rowReorder', this);
        /**
         * This event occurs when the user changes the column visibility, pinned option, or column width, or expands or collapses the column group from UI
         */
        this.columnChanged = new GridEvent('columnChanged', this);
        /**
         * This event occurs before editing, once editing has started, before editing has ended, or editing has stopped
         */
        this.editing = new GridEvent('editing', this);
        /**
         * This event occurs before loading next data range.
         */
        this.beforeLoadRange = new GridEvent('beforeLoadRange', this);
        this.container = container;
    }
    Object.defineProperty(DataViewComponent.prototype, "data", {
        /**
         * Specifies an array of objects for the grid model or a dataSource object
         */
        get: function () {
            return this._data;
        },
        set: function (value) {
            if (value === this._data) {
                return;
            }
            this._data = value;
            this.recreateView();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataViewComponent.prototype, "cols", {
        /**
         * Specifies an array of column definitions to initialize the grid
         */
        get: function () {
            return this._cols;
        },
        set: function (value) {
            if (_.isEqual(value, this._cols)) {
                return;
            }
            this._cols = value;
            if (this._initialized) {
                this.dataView.columns = value;
                this.dataView.invalidate();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataViewComponent.prototype, "config", {
        /**
         * Specifies a JSON object used to deserialize the grid from
         */
        get: function () {
            return this._config;
        },
        set: function (value) {
            if (value === this._config) {
                return;
            }
            this._config = value;
            this.recreateView();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataViewComponent.prototype, "layout", {
        /**
         * Specifies layout strategy. Default value is instance of {@link GridLayout} with default options
         */
        get: function () {
            return this._layout;
        },
        set: function (value) {
            if (value === this._layout) {
                return;
            }
            this._layout = value;
            this.recreateView();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataViewComponent.prototype, "options", {
        /**
         * Specifies options of layout strategy and other common options like filtering/sorting/grouping.
         */
        get: function () {
            return this._options;
        },
        set: function (value) {
            if (_.isEqual(value, this._options)) {
                return;
            }
            this._options = value;
            if (this.dataView && !_.isEmpty(this._options)) {
                assignGridOptions(this.dataView.options, this._options);
                this.dataView.invalidate();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataViewComponent.prototype, "groupingOptions", {
        /**
         * Specifies options of grouping strategy.
         */
        get: function () {
            return this._groupingOptions;
        },
        set: function (value) {
            if (_.isEqual(value, this._groupingOptions)) {
                return;
            }
            this._groupingOptions = value;
            if (this.dataView && !_.isEmpty(this._groupingOptions)) {
                assignDeep(this.dataView.layoutEngine.groupStrategy.options, this._groupingOptions);
                this.dataView.invalidate();
            }
        },
        enumerable: true,
        configurable: true
    });
    DataViewComponent.prototype.ngAfterViewInit = function () {
        this.createView();
        this._initialized = true;
        if (isTrue(this.autoFocus)) {
            this.container.nativeElement.focus();
        }
    };
    DataViewComponent.prototype.ngOnDestroy = function () {
        this.destroyView();
        this._initialized = false;
    };
    DataViewComponent.prototype.createView = function () {
        if (!this._layout) {
            this._layout = makeGridLayout(this._options || {});
        }
        this.dataView = new DataView(this.container.nativeElement, this.data, this.cols || this.config, this._layout);
        this.registerHandlers();
        setInstance(this.dataView.uid, this.dataView);
        this.created.emit(this.dataView);
        this.applyOptions();
    };
    DataViewComponent.prototype.destroyView = function () {
        if (this.dataView) {
            deleteInstance(this.dataView.uid);
            this.unregisterHandlers();
            this.dataView.destroy();
            this.dataView = undefined;
        }
    };
    DataViewComponent.prototype.recreateView = function () {
        if (!this._initialized) {
            return;
        }
        this.destroyView();
        this.createView();
    };
    DataViewComponent.prototype.registerHandlers = function () {
        var e_1, _a;
        try {
            for (var _b = __values(Object.keys(eventMap)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var eventName = _c.value;
                this[eventName].register();
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    DataViewComponent.prototype.unregisterHandlers = function () {
        var e_2, _a;
        try {
            for (var _b = __values(Object.keys(eventMap)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var eventName = _c.value;
                this[eventName].unregister();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    DataViewComponent.prototype.applyOptions = function (forceInvalidate) {
        if (forceInvalidate === void 0) { forceInvalidate = false; }
        var hasOptions = false;
        if (!_.isEmpty(this._options)) {
            hasOptions = true;
            assignGridOptions(this.dataView.options, this._options);
        }
        if (!_.isEmpty(this._groupingOptions) && this.dataView.layoutEngine.groupStrategy) {
            hasOptions = true;
            assignDeep(this.dataView.layoutEngine.groupStrategy.options, this._groupingOptions);
        }
        if (forceInvalidate || hasOptions) {
            this.dataView.invalidate();
        }
    };
    DataViewComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input()
    ], DataViewComponent.prototype, "data", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "cols", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "config", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "layout", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "options", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "groupingOptions", null);
    __decorate([
        Input()
    ], DataViewComponent.prototype, "autoFocus", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "created", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "selectionChanged", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "focusChanged", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "grouping", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "rowClick", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "rowDbClick", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "columnReorder", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "rowReorder", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "columnChanged", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "editing", void 0);
    __decorate([
        Output()
    ], DataViewComponent.prototype, "beforeLoadRange", void 0);
    DataViewComponent = __decorate([
        Component({
            selector: 'gc-dataview',
            template: "<ng-content></ng-content>",
            styles: [':host { display: block; width: 100%; height: 100% }']
        })
    ], DataViewComponent);
    return DataViewComponent;
}());
/**
 * Angular module with {@link DataViewComponent} included for importing in an angular application
 */
var DataViewModule = /** @class */ (function () {
    // tslint:disable-next-line:no-unnecessary-class
    function DataViewModule() {
    }
    DataViewModule = __decorate([
        NgModule({
            declarations: [DataViewComponent],
            exports: [DataViewComponent],
        })
        // tslint:disable-next-line:no-unnecessary-class
    ], DataViewModule);
    return DataViewModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { DataViewComponent, DataViewModule, ɵ0, ɵ1, ɵ2, ɵ3, ɵ4, ɵ5, ɵ6, ɵ7, ɵ8, ɵ9 };
//# sourceMappingURL=grapecity-dataviews.angular9.js.map
